@@ -32,7 +32,7 @@ import (
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/workqueue"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 	deschedulerapi "sigs.k8s.io/descheduler/pkg/api/v1alpha1"
 )
 
@@ -50,11 +50,12 @@ var validStrategies = sets.NewString(
 	"removepodsviolatingnodeaffinity",
 	"nodetaints",
 	"removepodsviolatingnodetaints",
-	"removepodshavingtoomanyrestarts")
+	"removepodshavingtoomanyrestarts",
+	"podlifetime")
 
 // deschedulerCommand provides descheduler command with policyconfigfile mounted as volume and log-level for backwards
 // compatibility with 3.11
-var DeschedulerCommand = []string{"/bin/descheduler", "--policy-config-file", "/policy-dir/policy.yaml", "--v", "5"}
+var DeschedulerCommand = []string{"/bin/descheduler", "--policy-config-file", "/policy-dir/policy.yaml", "--v", "2"}
 
 type TargetConfigReconciler struct {
 	ctx               context.Context
@@ -88,8 +89,9 @@ func NewTargetConfigReconciler(
 }
 
 func (c TargetConfigReconciler) sync() error {
-	descheduler, err := c.operatorClient.KubeDeschedulers(operatorclient.OperatorNamespace).Get(c.ctx, "cluster", metav1.GetOptions{})
+	descheduler, err := c.operatorClient.KubeDeschedulers(operatorclient.OperatorNamespace).Get(c.ctx, operatorclient.OperatorConfigName, metav1.GetOptions{})
 	if err != nil {
+		klog.ErrorS(err, "unable to get operator configuration", "namespace", operatorclient.OperatorNamespace, "kubedescheduler", operatorclient.OperatorConfigName)
 		return err
 	}
 
@@ -159,21 +161,101 @@ func (c *TargetConfigReconciler) manageConfigMap(descheduler *deschedulerv1beta1
 	return resourceapply.ApplyConfigMap(c.kubeClient.CoreV1(), c.eventRecorder, required)
 }
 
+func generateNamespaces(params []deschedulerv1beta1.Param) *deschedulerapi.Namespaces {
+	var namespaces deschedulerapi.Namespaces
+	for _, param := range params {
+		switch strings.ToLower(param.Name) {
+		case "includenamespaces":
+			namespaces.Include = strings.Split(param.Value, ",")
+		case "excludenamespaces":
+			namespaces.Exclude = strings.Split(param.Value, ",")
+		default:
+			klog.Warningf("unknown Namespaces value: %s", param.Name)
+		}
+	}
+	return &namespaces
+}
+
+func isPriorityThresholdParam(param deschedulerv1beta1.Param) bool {
+	switch strings.ToLower(param.Name) {
+	case "thresholdpriority", "thresholdpriorityclassname":
+		return true
+	}
+	return false
+}
+
+func generatePriorityThreshold(params []deschedulerv1beta1.Param) (string, *int32, error) {
+	var thresholdPriority *int32
+	var thresholdPriorityClassName string
+	for _, param := range params {
+		switch strings.ToLower(param.Name) {
+		case "thresholdpriority":
+			value, err := strconv.Atoi(param.Value)
+			if err != nil {
+				return "", nil, err
+			}
+			priority := int32(value)
+			thresholdPriority = &priority
+		case "thresholdpriorityclassname":
+			thresholdPriorityClassName = param.Value
+		default:
+			klog.Warningf("unknown PriorityThreshold value: %s", param.Name)
+		}
+	}
+	if len(thresholdPriorityClassName) > 0 && thresholdPriority != nil {
+		return "", nil, fmt.Errorf("cannot set both thresholdPriorityClassName and thresholdPriority")
+	}
+	return thresholdPriorityClassName, thresholdPriority, nil
+}
+
 // generateConfigMapString generates configmap needed for the string.
+// TODO(@damemi): Deprecate this in favor of an actual upstream Descheduler policy, see https://github.com/openshift/cluster-kube-descheduler-operator/issues/119
 func generateConfigMapString(requestedStrategies []deschedulerv1beta1.Strategy) (string, error) {
 	policy := &deschedulerapi.DeschedulerPolicy{Strategies: make(deschedulerapi.StrategyList)}
 	// There is no need to do validation here. By the time, we reach here, validation would have already happened.
 	for _, strategy := range requestedStrategies {
 		switch strings.ToLower(strategy.Name) {
 		case "duplicates", "removeduplicates":
-			policy.Strategies["RemoveDuplicates"] = deschedulerapi.DeschedulerStrategy{Enabled: true}
+			removeDuplicates := deschedulerapi.RemoveDuplicates{}
+			for _, param := range strategy.Params {
+				switch strings.ToLower(param.Name) {
+				case "excludeownerkinds":
+					removeDuplicates.ExcludeOwnerKinds = strings.Split(param.Value, ",")
+				}
+			}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
+			policy.Strategies["RemoveDuplicates"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
+				Params: &deschedulerapi.StrategyParameters{
+					RemoveDuplicates:           &removeDuplicates,
+					ThresholdPriority:          priority,
+					ThresholdPriorityClassName: priorityClassName,
+				},
+			}
+
 		case "interpodantiaffinity", "removepodsviolatinginterpodantiaffinity":
-			policy.Strategies["RemovePodsViolatingInterPodAntiAffinity"] = deschedulerapi.DeschedulerStrategy{Enabled: true}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
+			policy.Strategies["RemovePodsViolatingInterPodAntiAffinity"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
+				Params: &deschedulerapi.StrategyParameters{
+					ThresholdPriorityClassName: priorityClassName,
+					ThresholdPriority:          priority,
+					Namespaces:                 generateNamespaces(strategy.Params),
+				},
+			}
+
 		case "lownodeutilization":
 			utilizationThresholds := deschedulerapi.NodeResourceUtilizationThresholds{NumberOfNodes: 0}
 			thresholds := deschedulerapi.ResourceThresholds{}
 			targetThresholds := deschedulerapi.ResourceThresholds{}
 			for _, param := range strategy.Params {
+				if isPriorityThresholdParam(param) {
+					continue
+				}
 				value, err := strconv.Atoi(param.Value)
 				if err != nil {
 					return "", err
@@ -199,21 +281,47 @@ func generateConfigMapString(requestedStrategies []deschedulerv1beta1.Strategy)
 				utilizationThresholds.Thresholds = thresholds
 			}
 			if len(targetThresholds) > 0 {
-				utilizationThresholds.TargetThresholds = thresholds
+				utilizationThresholds.TargetThresholds = targetThresholds
+			}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
 			}
 			policy.Strategies["LowNodeUtilization"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
-				Params: deschedulerapi.StrategyParameters{
+				Params: &deschedulerapi.StrategyParameters{
 					NodeResourceUtilizationThresholds: &utilizationThresholds,
+					ThresholdPriorityClassName:        priorityClassName,
+					ThresholdPriority:                 priority,
 				},
 			}
+
 		case "nodeaffinity", "removepodsviolatingnodeaffinity":
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
 			policy.Strategies["RemovePodsViolatingNodeAffinity"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
-				Params: deschedulerapi.StrategyParameters{
-					NodeAffinityType: []string{"requiredDuringSchedulingIgnoredDuringExecution"},
+				Params: &deschedulerapi.StrategyParameters{
+					NodeAffinityType:           []string{"requiredDuringSchedulingIgnoredDuringExecution"},
+					ThresholdPriorityClassName: priorityClassName,
+					ThresholdPriority:          priority,
+					Namespaces:                 generateNamespaces(strategy.Params),
 				},
 			}
+
 		case "nodetaints", "removepodsviolatingnodetaints":
-			policy.Strategies["RemovePodsViolatingNodeTaints"] = deschedulerapi.DeschedulerStrategy{Enabled: true}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
+			policy.Strategies["RemovePodsViolatingNodeTaints"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
+				Params: &deschedulerapi.StrategyParameters{
+					ThresholdPriorityClassName: priorityClassName,
+					ThresholdPriority:          priority,
+					Namespaces:                 generateNamespaces(strategy.Params),
+				},
+			}
+
 		case "removepodshavingtoomanyrestarts":
 			podsHavingTooManyRestarts := deschedulerapi.PodsHavingTooManyRestarts{}
 			for _, param := range strategy.Params {
@@ -232,11 +340,45 @@ func generateConfigMapString(requestedStrategies []deschedulerv1beta1.Strategy)
 					podsHavingTooManyRestarts.IncludingInitContainers = value
 				}
 			}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
 			policy.Strategies["RemovePodsHavingTooManyRestarts"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
-				Params: deschedulerapi.StrategyParameters{
-					PodsHavingTooManyRestarts: &podsHavingTooManyRestarts,
+				Params: &deschedulerapi.StrategyParameters{
+					PodsHavingTooManyRestarts:  &podsHavingTooManyRestarts,
+					ThresholdPriorityClassName: priorityClassName,
+					ThresholdPriority:          priority,
+					Namespaces:                 generateNamespaces(strategy.Params),
 				},
 			}
+
+		case "podlifetime":
+			var lifetimeSeconds *uint
+			for _, param := range strategy.Params {
+				switch strings.ToLower(param.Name) {
+				case "maxpodlifetimeseconds":
+					value, err := strconv.Atoi(param.Value)
+					if err != nil {
+						return "", err
+					}
+					val := uint(value)
+					lifetimeSeconds = &val
+				}
+			}
+			priorityClassName, priority, err := generatePriorityThreshold(strategy.Params)
+			if err != nil {
+				return "", err
+			}
+			policy.Strategies["PodLifeTime"] = deschedulerapi.DeschedulerStrategy{Enabled: true,
+				Params: &deschedulerapi.StrategyParameters{
+					MaxPodLifeTimeSeconds:      lifetimeSeconds,
+					ThresholdPriorityClassName: priorityClassName,
+					ThresholdPriority:          priority,
+					Namespaces:                 generateNamespaces(strategy.Params),
+				},
+			}
+
 		default:
 			klog.Warningf("not using unknown strategy '%s'", strategy.Name)
 		}
